/*
 * ColorSudoku.c
 *
 * Created: 01.05.2012 09:19:45
 *  Author: Liz
 */ 

#define F_CPU 16000000

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <stdlib.h>
#include "integer.h"
#include <avr/pgmspace.h>




#define clear_bit(reg, bit) (reg&=~(1<<bit))
#define set_bit(reg, bit) (reg|= (1<<bit))
#define is_set(reg, bit) ((reg & (1 << bit)))

enum difficulty {EASY_DIFF = 0, MEDIUM_DIFF, HARD_DIFF};

struct rgb {
	
	BYTE r;
	BYTE g;
	BYTE b;
};


#include "spi_lcd.h"
#include "pwm_rgb.h"



#define DIM_SQUARE 13
#define SQUARE_INCREMENT 15 //DIM_SQUARE + DIM_SMALL_BORDER
#define DIM_BIG_BORDER 3
#define DIM_SMALL_BORDER 2
struct rgb background_color = {255,255,255};
struct rgb big_border_color = {0,0,0};
struct rgb small_border_color = {180,180,180};

//traduc in culori doar cand e vorba sa afisez
//aceasta matrice va avea drept corespondent o matrice de struct rgb, de 132 X 132 pixeli
//sau nu tin matrice, ci transmit direct; tin minte doar niste offset-uri - cum ar fi patratul curent etc

#define NR_OF_GAMES 2

const BYTE all_games[3][NR_OF_GAMES][8][8] =
	{		
		{	
			{
				{0,0,3,0,0,0,4,0},
				{2,0,0,0,5,3,0,0},
				{0,5,2,0,3,6,0,4},
				{0,0,0,0,1,5,0,8},
				{0,4,5,0,0,1,0,2},
				{8,6,1,2,0,0,5,0},
				{4,3,7,5,6,2,0,0},
				{6,0,8,0,0,7,3,0}
			},	
			{
				{0,3,0,0,1,0,0,0},
				{1,6,7,0,5,2,3,4},
				{0,0,0,7,2,4,8,0},
				{2,4,8,6,0,0,0,0},
				{4,0,5,0,0,0,3,0},
				{0,8,0,1,0,7,5,2},
				{7,0,0,5,0,0,0,3},
				{0,0,6,3,7,0,0,5}
			}
		},

		{
			{
				{0,8,0,2,5,0,0,0},
				{0,0,0,0,0,0,0,0},
				{8,6,3,0,0,0,0,4},
				{0,0,5,0,0,6,0,0},
				{2,0,6,0,0,0,3,0},
				{0,0,0,0,0,0,0,0},
				{0,0,2,0,0,0,0,0},
				{3,0,7,0,0,0,2,0}
			},
			{
				{0,0,0,0,0,0,0,0},
				{0,3,0,0,0,0,0,2},
				{0,7,0,0,6,0,0,0},
				{0,0,0,5,1,0,8,0},
				{0,4,0,0,0,0,0,0},
				{0,0,0,8,0,0,4,0},
				{5,0,2,0,0,8,0,0},
				{0,8,3,0,5,0,0,0}
			}
		},
		{
			{
				{0,0,0,4,0,3,0,0},
				{8,0,1,7,0,0,0,0},
				{0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0},
				{0,0,3,0,0,0,8,0},
				{0,0,0,0,0,6,0,0},
				{0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0}
			},
			{
				{0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,4,6},
				{0,0,4,0,8,0,5,3},
				{0,0,0,0,0,0,0,0},
				{0,0,0,0,0,0,0,0},
				{0,0,8,0,0,0,0,0},
				{4,0,0,0,0,0,0,0}
			}
		}
	}
;

#define NR_COLOR_SCHEMES_PER_GAME 3

const struct rgb all_color_schemes[3][NR_COLOR_SCHEMES_PER_GAME][8] =
{
	{	
		{
			{255,0,0},
			{0,255,0},
			{0,0,255},
			{255,255,0},
			{255,0,255},
			{0,255,255},
			{255,125,125},
			{150,0,255}
		},	
		{
			{128,0,0},
			{50,205,50},
			{65,105,255},
			{255,215,0},
			{221,160,221},
			{127,255,212},
			{255,127,80},
			{112,128,144}
		},
		{
			{255,140,0},
			{0,255,127},
			{0,191,255},
			{255,105,180},
			{64,224,208},
			{148,0,211},
			{105,105,105},
			{95,158,160}
		}
	},
	{	

		{
			{255,99,71},
			{250,128,114},
			{46,139,87},
			{60,179,113},
			{30,144,255},
			{0,191,255},
			{255,215,0},
			{255,255,0}
		},
		{
			{210,105,30},
			{160,82,45},
			{176,196,222},
			{100,149,237},
			{218,165,32},
			{255,215,0},
			{186,85,211},
			{148,0,211}
		},
		{
			{205,92,92},
			{220,20,60},
			{154,205,50},
			{143,188,143},
			{0,0,139},
			{0,0,205},
			{95,158,160},
			{119,136,153}
		}
	},
	{
		{
			{178,34,34},
			{220,20,60},
			{255,0,0},
			{128,0,0},
			{85,107,47},
			{107,142,35},
			{0,128,0},
			{46,139,87}
		},
		{
			{240,128,128},
			{233,150,122},
			{250,128,114},
			{255,160,122},
			{0,206,209},
			{64,224,208},
			{32,178,170},
			{102,205,170}
		},
		{
			{128,128,2},
			{169,169,169},
			{192,192,192},
			{220,220,220},
			{75,0,130},
			{72,61,139},
			{106,90,205},
			{123,104,238}
		}
	}
};

int crt_game_difficulty;
int crt_no_game;
int crt_no_color_scheme;

#define game_orig all_games[crt_game_difficulty][crt_no_game]
#define color_scheme all_color_schemes[crt_game_difficulty][crt_no_color_scheme]

BYTE game[8][8];
BYTE no_squares = 0;


#define draw_background() (draw_rectangle(0,0,132,132,0,background_color))
#define GRID_XS 4
#define GRID_YS 4
#define GRID_WIDTH 124
#define GRID_HEIGHT 124

BYTE x_crt = 1;
BYTE y_crt = 1;
BYTE x_prev = 0;
BYTE y_prev = 0;
BYTE x_sel_square = GRID_XS + DIM_BIG_BORDER;
BYTE y_sel_square = GRID_YS + DIM_BIG_BORDER;
BYTE x_sel_square_prev =  0;
BYTE y_sel_square_prev =  0;
BYTE selected_color;

void load_game() {

	int ii,jj;
	for (ii = 0 ; ii < 8; ii++)
		for (jj = 0 ; jj < 8 ; jj++) {
			game[ii][jj] = game_orig[ii][jj];
			no_squares++;
		}	
		
		 x_crt = 1;
		 y_crt = 1;
		 x_prev = 0;
		 y_prev = 0;
		 x_sel_square = GRID_XS + DIM_BIG_BORDER;
		 y_sel_square = GRID_YS + DIM_BIG_BORDER;
		 x_sel_square_prev =  0;
		 y_sel_square_prev =  0;
		 selected_color = 0;		
}

void load_color_scheme(int i, int j) {

	crt_game_difficulty = i;
	crt_no_color_scheme = j;	
}


//umple patratul selectat cu o culoare
//trebuie sa actualizez si in matricea mare
#define color_sel_square(color) (draw_rectangle(x_sel_square, y_sel_square, DIM_SQUARE, DIM_SQUARE, 0, ((struct rgb)color)))
#define fill_square(xs,ys,color) (draw_rectangle(xs, ys, DIM_SQUARE, DIM_SQUARE, 0, ((struct rgb)color)))

void draw_game_grid() {
	
	draw_background();
	
	//liniile interioare
	//comanda adrese pe verticala
	//sendCMD(MEMORY_DATA_ACCESS_CONTROL);
	//sendData(8|32);
	
	BYTE offset_xs = GRID_XS + DIM_BIG_BORDER + DIM_SQUARE;
	BYTE offset_linie_mijloc;
	
	draw_line(offset_xs, GRID_YS, DIM_SMALL_BORDER, GRID_HEIGHT, small_border_color);
	offset_xs += SQUARE_INCREMENT;
	draw_line(offset_xs, GRID_YS, DIM_SMALL_BORDER, GRID_HEIGHT, small_border_color);
	offset_xs += SQUARE_INCREMENT;
	draw_line(offset_xs, GRID_YS, DIM_SMALL_BORDER, GRID_HEIGHT, small_border_color);
	offset_xs += SQUARE_INCREMENT;
	offset_linie_mijloc = offset_xs;
//	draw_line(offset_xs, GRID_YS, DIM_SMALL_BORDER, GRID_HEIGHT, big_border_color);
//	draw_line(offset_linie_mijloc, GRID_YS, DIM_SMALL_BORDER, GRID_HEIGHT, big_border_color);
	offset_xs += SQUARE_INCREMENT;
	draw_line(offset_xs, GRID_YS, DIM_SMALL_BORDER, GRID_HEIGHT, small_border_color);
	offset_xs += SQUARE_INCREMENT;
	draw_line(offset_xs, GRID_YS, DIM_SMALL_BORDER, GRID_HEIGHT, small_border_color);
	offset_xs += SQUARE_INCREMENT;
	draw_line(offset_xs, GRID_YS, DIM_SMALL_BORDER, GRID_HEIGHT, small_border_color);

	//revenire la adrese pe orizontala
	sendCMD(MEMORY_DATA_ACCESS_CONTROL);
	sendData(8);
	
	BYTE offset_ys = GRID_YS + DIM_BIG_BORDER + DIM_SQUARE;
	
	draw_line(GRID_XS, offset_ys, GRID_WIDTH, DIM_SMALL_BORDER, small_border_color);
	offset_ys += SQUARE_INCREMENT;
	draw_line(GRID_XS, offset_ys, GRID_WIDTH, DIM_SMALL_BORDER, big_border_color);
	offset_ys += SQUARE_INCREMENT;
	draw_line(GRID_XS, offset_ys, GRID_WIDTH, DIM_SMALL_BORDER, small_border_color);
	offset_ys += SQUARE_INCREMENT;
	draw_line(GRID_XS, offset_ys, GRID_WIDTH, DIM_SMALL_BORDER, big_border_color);
	offset_ys += SQUARE_INCREMENT;
	draw_line(GRID_XS, offset_ys, GRID_WIDTH, DIM_SMALL_BORDER, small_border_color);
	offset_ys += SQUARE_INCREMENT;
	draw_line(GRID_XS, offset_ys, GRID_WIDTH, DIM_SMALL_BORDER, big_border_color);
	offset_ys += SQUARE_INCREMENT;
	draw_line(GRID_XS, offset_ys, GRID_WIDTH, DIM_SMALL_BORDER, small_border_color);

	//conturul exterior pentru grid
	draw_rectangle(GRID_XS, GRID_YS, GRID_WIDTH, GRID_HEIGHT, DIM_BIG_BORDER, big_border_color);
	
	//comanda adrese pe verticala
	//sendCMD(MEMORY_DATA_ACCESS_CONTROL);
	//sendData(8|32);
	//draw_line(offset_linie_mijloc, GRID_YS, DIM_SMALL_BORDER, GRID_HEIGHT, big_border_color);
	draw_rectangle(offset_linie_mijloc, GRID_YS, DIM_SMALL_BORDER, GRID_HEIGHT, 0, big_border_color);
	
	//revenire la adrese pe orizontala
//	sendCMD(MEMORY_DATA_ACCESS_CONTROL);
//	sendData(8);
}



void draw_game() {
	
	BYTE xs = GRID_XS + DIM_BIG_BORDER, ys = GRID_YS + DIM_BIG_BORDER;
	
	//in functie de joc si de schema de culori, plasez culorile initiale pe grid
	int i,j;
	for (i = 0 ; i < 8; i++) {
		
		xs = GRID_XS + DIM_BIG_BORDER;
		
		for (j = 0 ; j < 8; j++) {
			
			if (game[i][j] != 0) {
				fill_square(xs, ys, color_scheme[game[i][j]-1]);
				
			}				
				
			xs += SQUARE_INCREMENT;
		}
		ys += SQUARE_INCREMENT;		
	}					
}


//nu trebuie sa pun culori decat in patratele goale !!!!!!!!!!!!

#define SEL_SQUARE_OUTLINE_THICKNESS 3
struct rgb sel_square_outline_color = {0,0,0};

void place_sel_square() {
	
	//ASTA NU E OK
	//nu stiu cum sa fac sa revin la conturul anterior - trebuie sa verific in ce patrat ma aflu si in functie de asta sa refac liniile...
	
	//mai e un caz in care se redeseneaza - cand revin din meniul GAME
	//!!!!!!!!!!!!!!!!!!!!!!!
	//trebuie sa setez x_sel_square_prev...
	
	
	//sterge conturul anterior
	if (x_sel_square != x_sel_square_prev || y_sel_square != y_sel_square_prev) {
			
				
					
		if (x_sel_square_prev != 0 && y_sel_square_prev != 0) {
		
			draw_rectangle( y_sel_square_prev-2, x_sel_square_prev-2,DIM_SQUARE + 4, DIM_SQUARE + 4, DIM_SMALL_BORDER, small_border_color);
			//daca e la o margine, atunci reface liniile negre
			
			
			//stanga
			if (x_prev == 1 || x_prev == 5) 
				draw_line(x_sel_square_prev-2,y_sel_square_prev - 2, DIM_SMALL_BORDER, DIM_SQUARE + 4, big_border_color);
		
		
			//dreapta
			if ( x_prev == 4 || x_prev == 8) 
				draw_line(x_sel_square_prev-2 + SQUARE_INCREMENT,  y_sel_square_prev - 2, DIM_SMALL_BORDER, DIM_SQUARE + 4, big_border_color);
		
			//jos
			if (y_prev == 2 || y_prev == 4 || y_prev == 6 || y_prev == 8) 
				draw_line(x_sel_square_prev,  y_sel_square_prev - 2 + SQUARE_INCREMENT, DIM_SQUARE + 4, DIM_SMALL_BORDER, big_border_color);  
				
			
			//sus	
			if (y_prev == 1 || y_prev == 3 || y_prev == 5 || y_prev == 7) 
				draw_line( x_sel_square_prev-2, y_sel_square_prev -2, DIM_SQUARE + 4, DIM_SMALL_BORDER, big_border_color);		
			
			
			//umple apoi cu culoarea anterioara
			if(game[y_prev-1][x_prev-1])
				fill_square(x_sel_square_prev, y_sel_square_prev, color_scheme[game[y_prev-1][x_prev-1]-1]);
			else 
				fill_square(x_sel_square_prev, y_sel_square_prev, background_color);		
		
		}
		
										
		//muta conturul de selectie
		draw_rectangle( y_sel_square-2, x_sel_square-2,DIM_SQUARE + 4, DIM_SQUARE + 4, SEL_SQUARE_OUTLINE_THICKNESS, sel_square_outline_color);
			
	
		x_sel_square_prev = x_sel_square;
		y_sel_square_prev = y_sel_square;	
		x_prev = x_crt;
		y_prev = y_crt;	
	}	
}



//!!!!!!!!!!!!!!!!!!!!!!!
// X e orizontala - defineste coloanele
// y e verticala - defineste randurile
// (0,0) e in coltul stanga sus (in RAM cel putin)

//de unde incepe (0.0) pe display efectiv - e nevoie de inversare?

#define go_up() ({if (y_crt > 1) {y_sel_square -= SQUARE_INCREMENT; y_crt--; place_sel_square();}})
#define go_down() ({if (y_crt < 8) {y_sel_square += SQUARE_INCREMENT; y_crt++; place_sel_square();}})
#define go_left() ({if (x_crt > 1) {x_sel_square -= SQUARE_INCREMENT; x_crt--; place_sel_square();}})
#define go_right() ({if (x_crt < 8) {x_sel_square += SQUARE_INCREMENT; x_crt++; place_sel_square();}})

//TODO
//pwm - cum aleg culorile si cum mentin pwm-ul (optional)

//MENIU

//COLOR SUDOKU

//START GAME
//CHOOSE DIFFICULTY
//CHOOSE COLOR SCHEME
//CUSTOM (optional)


BYTE crt_state;
BYTE crt_menu;


enum menus {MAIN_MENU, DIFFICULTY_MENU, COLOR_SCHEME_MENU, GAME_MENU, CHOOSE_COLOR_MENU};

#define OPTION_START_XS   20
#define OPTION_START_YS   60
#define OPTION_INCREMENT  20
BYTE cursor_xs = OPTION_START_XS - 15;
BYTE cursor_ys = OPTION_START_YS;
#define NR_OPTIONS 3
BYTE crt_option;

enum options { START_GAME, DIFFICULTY, COLOR_SCHEME, EASY, MEDIUM, HARD, CHOOSE_COLOR, RESTART_GAME, MAIN_MENU_OPTION, FIRST_COLOR_SCH, SECOND_COLOR_SCH, THIRD_COLOR_SCH};

struct rgb menu_back_color = {0,200,100};

//variabile: cursor_xs, cursor_ys, NR_OPTIONS, OPTION_INCREMENT, LAST_OPTION

void display_cursor() {
	
	struct rgb font_color = {0, 80, 150};
	LCDPutChar('>',cursor_xs, cursor_ys, MEDIUM_FONT, font_color, menu_back_color);	
}

void move_cursor_down() {
	
	//se reface background-ul
	LCDPutChar('>',cursor_xs, cursor_ys, MEDIUM_FONT, menu_back_color, menu_back_color);
	if (crt_option == COLOR_SCHEME || crt_option == MAIN_MENU_OPTION || crt_option == THIRD_COLOR_SCH || crt_option == HARD)
	{
		cursor_ys = OPTION_START_YS;
		crt_option -= (NR_OPTIONS-1);
	}		
	else	
	{	
		cursor_ys +=  OPTION_INCREMENT;
		crt_option++;
	}
	display_cursor();		
}

void move_cursor_up() {
	
	//se reface background-ul
	LCDPutChar('>',cursor_xs, cursor_ys, MEDIUM_FONT, menu_back_color, menu_back_color);
	if (crt_option == START_GAME || crt_option == EASY || crt_option == FIRST_COLOR_SCH || crt_option == CHOOSE_COLOR)
	{
		cursor_ys = OPTION_START_YS + 2 * OPTION_INCREMENT;
		crt_option += (NR_OPTIONS-1);
	}		
	else	
	{	
		cursor_ys -=  OPTION_INCREMENT;
		crt_option--;
	}		
	display_cursor();
}

void display_menu(char * title, char * opt1, char * opt2, char * opt3) {
	
	//background pt meniu
	
	draw_rectangle(0,0,132,132,0,menu_back_color);
	
	//optiuni
	struct rgb font_color = {150, 120, 80};
	LCDPutStr(title, 18, 30, MEDIUM_FONT, font_color, menu_back_color);
	font_color.r = 0;
	font_color.g = 0;
	font_color.b = 0;
	LCDPutStr(opt1, OPTION_START_XS, OPTION_START_YS, MEDIUM_FONT, font_color, menu_back_color);	
	LCDPutStr(opt2, OPTION_START_XS, OPTION_START_YS + OPTION_INCREMENT, MEDIUM_FONT, font_color, menu_back_color);
	LCDPutStr(opt3, OPTION_START_XS, OPTION_START_YS + 2*OPTION_INCREMENT, MEDIUM_FONT, font_color, menu_back_color);
	display_cursor();
}

#define restore_cursor() ({cursor_xs = OPTION_START_XS - 15; cursor_ys = OPTION_START_YS;})
#define main_menu() ({crt_menu = MAIN_MENU; crt_option = START_GAME; restore_cursor(); display_menu("COLOR SUDOKU", "Start Game", "Difficulty", "Color Scheme");})
#define game_menu() ({crt_menu = GAME_MENU; crt_option = CHOOSE_COLOR; restore_cursor(); display_menu("GAME OPTIONS", "Choose Color", "Restart", "Main Menu");})
#define difficulty_menu() ({crt_menu = DIFFICULTY_MENU; crt_option = EASY; restore_cursor(); display_menu("DIFFICULTY", "Easy", "Medium", "Hard");})

//NR_COLOR_SCHEMES_PER_GAME = NR_OPTIONS

//folosesc acelasi display_cursor, mai putin la choose_color_menu

void color_scheme_menu() {
	
	//background pt meniu
	
	crt_menu = COLOR_SCHEME_MENU;
	crt_option = FIRST_COLOR_SCH;
	restore_cursor();
	
	draw_rectangle(0,0,132,132,0,menu_back_color);
	
	//optiuni
	struct rgb font_color = {150, 120, 80};
	LCDPutStr("COLOR SCHEME", 18, 30, MEDIUM_FONT, font_color, menu_back_color);
	
	int i,j;
	BYTE start_ys = OPTION_START_YS;
	
	for (i = 0 ; i < NR_COLOR_SCHEMES_PER_GAME; i++) {
		
		BYTE start_xs = OPTION_START_XS - 5;
		
		for (j = 0 ; j < 8; j++) {
		
			fill_square(start_xs-1, start_ys, all_color_schemes[crt_game_difficulty][i][j]);
			start_xs += SQUARE_INCREMENT;
		}
		
		start_ys += OPTION_INCREMENT;
	}		
	
	display_cursor();
}




void choose_color_menu() {
	
	crt_menu = CHOOSE_COLOR_MENU;
	
	//desenez cele 8 culori si ma plimb cu patratul pe ele ca sa selectez...
	draw_rectangle(0,0,132,132,0,menu_back_color);
	
	//optiuni
	struct rgb font_color = {150, 120, 80};
	LCDPutStr("CHOOSE COLOR", 18, 30, MEDIUM_FONT, font_color, menu_back_color);
	
	int i;
	BYTE start_xs = OPTION_START_XS- 10;
	for (i = 0; i < 8; i++) {
		
		fill_square(start_xs - 1, OPTION_START_YS+OPTION_INCREMENT, color_scheme[i]);
		start_xs += SQUARE_INCREMENT;
	}	
	draw_rectangle(  OPTION_START_YS + OPTION_INCREMENT - 2, OPTION_START_XS - 10 + selected_color*SQUARE_INCREMENT - 2,DIM_SQUARE+4, DIM_SQUARE+4, 2, big_border_color);	
}


//stari
enum states {GAME_STATE, REGULAR_MENU_STATE, CHOOSE_COLOR_STATE};

//testez enter si reset
//la enter setez variabile, in functie de meniu si de optiune - crt_option - toate optiunile posibile...
//la reset - activez game_menu sau ma intorc la joc

#define display_game() ({draw_game_grid(); draw_game(); place_sel_square();})


int main(void)
{
	
	init_lcd();
	init_pwm();	
	init_dir_buttons();
	crt_state = REGULAR_MENU_STATE;
	crt_no_game = -1;
	crt_game_difficulty = EASY_DIFF;
	crt_no_color_scheme = 0;
	main_menu();
	
	_delay_ms(100);
		
	//busy waiting la greu; i hope that's not a problem
    while(1)
    {
		switch(crt_state) {
			
			case REGULAR_MENU_STATE: {
				
				if (!is_set(PINC, ENTER)) {
					
					//inseamna ca s-a selectat o optiune
					//verific ce optiune s-a selectat
					
					switch(crt_option) {
						
						case START_GAME: {
			
							crt_no_game++;
							if (crt_no_game == NR_OF_GAMES)
								crt_no_game = 0;
							load_game();
							display_game();	
							crt_state = GAME_STATE;			
						} break;
						
						case DIFFICULTY: {
							
							difficulty_menu();
						} break;
						
						case COLOR_SCHEME: {
							
							color_scheme_menu();
						} break;
						
						case CHOOSE_COLOR: {
							
							choose_color_menu();
							crt_state = CHOOSE_COLOR_STATE;
						} break;
						
						case RESTART_GAME: {
							
							//crt_game_diificulty si crt_no_game raman neschimbate
							load_game();
							display_game();
							crt_state = GAME_STATE;
						} break;
						
						case MAIN_MENU_OPTION: {
							
							main_menu();
						} break;
						
						case EASY: {
							
							crt_game_difficulty = EASY_DIFF;
							main_menu();
						} break;
						
						case MEDIUM: {
							
							crt_game_difficulty = MEDIUM_DIFF;
							main_menu();
						} break;
						
						case HARD: {
							
							crt_game_difficulty = HARD_DIFF;
							main_menu();
						} break;
						
						case FIRST_COLOR_SCH: {
							
							crt_no_color_scheme = 0;
							main_menu();
						} break;
						
						case SECOND_COLOR_SCH: {
							
							crt_no_color_scheme= 1;
							main_menu();
						} break;
						
						case THIRD_COLOR_SCH: {
							
							crt_no_color_scheme = 2;
							main_menu();
						} break;
						
					}
						
							set_bit(PINC, ENTER);		
				}				
											
				if (!is_set(PINC, RESET_BUTTON)) {
					
					switch(crt_menu) {
						
						case MAIN_MENU: {
							//nothing
						} break;
						
						case DIFFICULTY_MENU: {
							//se revine la main menu
							main_menu();
						} break;
						
						case COLOR_SCHEME_MENU: {
							
							//se revine la main menu
							main_menu();
						} break;
						
						case CHOOSE_COLOR_MENU: {
							
							//din meniul choose color se revine la meniul anterior, game menu
							game_menu();
						} break;
						
						case GAME_MENU: {
							
							//se revine la joc
							display_game();
							crt_state = GAME_STATE;
						} break;						
					}					
				}
				
				if (!is_set(PINC, UP)) {
					
					move_cursor_up();
				}
				if (!is_set(PINC, DOWN)) {
					
					move_cursor_down();
				}
				
			}	break;
			
			//afiseaza prost
			case GAME_STATE: {
					
					//butonul enter se comenteaza daca suntem in mod PWM !!!!!!!!!!!!!!				
				if (!is_set(PINC, ENTER)) {					
					
					//nu are voie sa suprascrie culorile initiale
					if (game_orig[y_crt-1][x_crt-1] == 0)
					{
						color_sel_square(color_scheme[selected_color]);
						//muta conturul de selectie
						draw_rectangle( y_sel_square-2, x_sel_square-2,DIM_SQUARE + 4, DIM_SQUARE + 4, SEL_SQUARE_OUTLINE_THICKNESS, sel_square_outline_color);
		
						game[y_crt-1][x_crt-1] = selected_color + 1;
						no_squares++;
					}					
				}
				if (!is_set(PINC, RESET_BUTTON)) {
				
					game_menu();
					crt_state = REGULAR_MENU_STATE;
				}				
				
				if (!is_set(PINC, UP)) {
					
					go_up();
				}
				if (!is_set(PINC, DOWN)) {
					
					go_down();
				}
				if (!is_set(PINC, LEFT)) {
					
					go_left();
				}
				if (!is_set(PINC, RIGHT)) {
					
					go_right();
				}
				
				//PWM
				//logica directa sau inversa la codificator?
						//in choose_color trebuie sa verific ce led a fost apasat
				/*
				BYTE modified = 0;
				if (is_set(PORT_LED_BUTTONS, LED_BUTTON0)) {set_bit(nr_led, 0); modified = 1;}
				if (is_set(PORT_LED_BUTTONS, LED_BUTTON1)) {set_bit(nr_led, 1); modified = 1;}
				if (is_set(PORT_LED_BUTTONS, LED_BUTTON2)) {set_bit(nr_led, 2); modified = 1;}
		
				if (modified) {
			
					selected_color = nr_led;
					//pot sa zic direct si c;ear_bit(PINC, ENTER); si se ocupa tot game states dupa aia
					color_sel_square(color_scheme[selected_color]);		
					game[y_crt][x_crt] = selected_color + 1;				
					no_squares++;	
				}			
				*/
				
				//verific daca s-a terminat jocul
				if (no_squares == 64) {
					
					int i,j; 
					BYTE ref_sum = 36;
					BYTE win = 1;
					for (i = 0 ; i < 8; i++) {
						
						BYTE sum = 0;
						for (j = 0 ; j < 8; j++) 
							sum += game[i][j];
						if (sum != ref_sum) {
							win = 0;
							break;
						}
					}				
					
					if (win) {
						
						struct rgb color = {255,0,0};
						draw_rectangle(0,0,132,132,0,menu_back_color);
						LCDPutStr("CONGRATS!", 40, 60, MEDIUM_FONT, color, menu_back_color);
						_delay_ms(1000);
						main_menu();
						crt_state = REGULAR_MENU_STATE;
					}						
				}				
			}	break;
			
			case CHOOSE_COLOR_STATE: {
																						
				if (!is_set(PINC, ENTER)) {
					
					display_game();
					//muta conturul de selectie
					draw_rectangle( y_sel_square-2, x_sel_square-2,DIM_SQUARE + 4, DIM_SQUARE + 4, SEL_SQUARE_OUTLINE_THICKNESS, sel_square_outline_color);
		
					crt_state = GAME_STATE;
				}
				if (!is_set(PINC, RESET_BUTTON)) {
					
					//se revine la meniul anterior
				//	game_menu();
					crt_state = REGULAR_MENU_STATE;
				}
				
				//la up si down nu se intampla nimic
				
				//la left si right si decrementeaza / incrementeaza nr optiunii 
				if (!is_set(PINC, LEFT)) {
					
					if (selected_color > 0) {
					//redeseneaza cursorul de selectie
						draw_rectangle(  OPTION_START_YS + OPTION_INCREMENT - 2, OPTION_START_XS - 10 + selected_color*SQUARE_INCREMENT - 2,DIM_SQUARE+4, DIM_SQUARE+4, 2, menu_back_color);
						selected_color--;
						draw_rectangle(  OPTION_START_YS + OPTION_INCREMENT - 2, OPTION_START_XS - 10 + selected_color*SQUARE_INCREMENT - 2,DIM_SQUARE+4, DIM_SQUARE+4, 2, big_border_color);
					}					
				}
				if (!is_set(PINC, RIGHT)) {
					
					if (selected_color < 7) {

						draw_rectangle(  OPTION_START_YS + OPTION_INCREMENT - 2, OPTION_START_XS - 10 + selected_color*SQUARE_INCREMENT - 2,DIM_SQUARE+4, DIM_SQUARE+4, 2, menu_back_color);
						selected_color++;
						draw_rectangle(  OPTION_START_YS + OPTION_INCREMENT - 2, OPTION_START_XS - 10 + selected_color*SQUARE_INCREMENT - 2,DIM_SQUARE+4, DIM_SQUARE+4, 2, big_border_color);	
					}					
					//redeseneaza cursorul de selectie
				}
			}	
		}	
		
		//tin led-urile aprinse tot timpul? as putea...
		//aici PWM
		int i;
		for (i = 0; i < 8; i++) {
		
			light_up_LED(i,color_scheme[i].r, color_scheme[i].g, color_scheme[i].b);
		}							
				
    }	
}

//mesaj de win la sfarsit